@@ -1,5 +1,13 @@
 use {super::*, serde::Serialize};
 
+#[derive(Debug)]
+struct Invocation<'src: 'run, 'run> {
+  arguments: &'run [&'run str],
+  recipe: &'run Recipe<'src>,
+  settings: &'run Settings<'src>,
+  scope: &'run Scope<'src, 'run>,
+}
+
 #[derive(Debug, PartialEq, Serialize)]
 pub(crate) struct Justfile<'src> {
   pub(crate) aliases: Table<'src, Alias<'src>>,
@@ -8,6 +16,7 @@ pub(crate) struct Justfile<'src> {
   pub(crate) default: Option<Rc<Recipe<'src>>>,
   #[serde(skip)]
   pub(crate) loaded: Vec<PathBuf>,
+  pub(crate) modules: BTreeMap<String, Justfile<'src>>,
   pub(crate) recipes: Table<'src, Rc<Recipe<'src>>>,
   pub(crate) settings: Settings<'src>,
   pub(crate) warnings: Vec<Warning>,
@@ -67,6 +76,44 @@ impl<'src> Justfile<'src> {
       .next()
   }
 
+  fn scope<'run>(
+    &'run self,
+    config: &'run Config,
+    dotenv: &'run BTreeMap<String, String>,
+    search: &'run Search,
+    overrides: &BTreeMap<String, String>,
+    parent: &'run Scope<'src, 'run>,
+  ) -> RunResult<'src, Scope<'src, 'run>>
+  where
+    'src: 'run,
+  {
+    let mut scope = parent.child();
+    let mut unknown_overrides = Vec::new();
+
+    for (name, value) in overrides {
+      if let Some(assignment) = self.assignments.get(name) {
+        scope.bind(assignment.export, assignment.name, value.clone());
+      } else {
+        unknown_overrides.push(name.clone());
+      }
+    }
+
+    if !unknown_overrides.is_empty() {
+      return Err(Error::UnknownOverrides {
+        overrides: unknown_overrides,
+      });
+    }
+
+    Evaluator::evaluate_assignments(
+      &self.assignments,
+      config,
+      dotenv,
+      scope,
+      &self.settings,
+      search,
+    )
+  }
+
   pub(crate) fn run(
     &self,
     config: &Config,
@@ -92,33 +139,9 @@ impl<'src> Justfile<'src> {
       BTreeMap::new()
     };
 
-    let scope = {
-      let mut scope = Scope::new();
-      let mut unknown_overrides = Vec::new();
+    let root = Scope::new();
 
-      for (name, value) in overrides {
-        if let Some(assignment) = self.assignments.get(name) {
-          scope.bind(assignment.export, assignment.name, value.clone());
-        } else {
-          unknown_overrides.push(name.clone());
-        }
-      }
-
-      if !unknown_overrides.is_empty() {
-        return Err(Error::UnknownOverrides {
-          overrides: unknown_overrides,
-        });
-      }
-
-      Evaluator::evaluate_assignments(
-        &self.assignments,
-        config,
-        &dotenv,
-        scope,
-        &self.settings,
-        search,
-      )?
-    };
+    let scope = self.scope(config, &dotenv, search, overrides, &root)?;
 
     match &config.subcommand {
       Subcommand::Command {
@@ -193,13 +216,7 @@ impl<'src> Justfile<'src> {
     let argvec: Vec<&str> = if !arguments.is_empty() {
       arguments.iter().map(String::as_str).collect()
     } else if let Some(recipe) = &self.default {
-      let min_arguments = recipe.min_arguments();
-      if min_arguments > 0 {
-        return Err(Error::DefaultRecipeRequiresArguments {
-          recipe: recipe.name.lexeme(),
-          min_arguments,
-        });
-      }
+      recipe.check_can_be_default_recipe()?;
       vec![recipe.name()]
     } else if self.recipes.is_empty() {
       return Err(Error::NoRecipes);
@@ -209,33 +226,31 @@ impl<'src> Justfile<'src> {
 
     let arguments = argvec.as_slice();
 
-    let mut missing = vec![];
-    let mut grouped = vec![];
-    let mut rest = arguments;
+    let mut missing = Vec::new();
+    let mut invocations = Vec::new();
+    let mut remaining = arguments;
+    let mut scopes = BTreeMap::new();
+    let arena: Arena<Scope> = Arena::new();
 
-    while let Some((argument, mut tail)) = rest.split_first() {
-      if let Some(recipe) = self.get_recipe(argument) {
-        if recipe.parameters.is_empty() {
-          grouped.push((recipe, &[][..]));
-        } else {
-          let argument_range = recipe.argument_range();
-          let argument_count = cmp::min(tail.len(), recipe.max_arguments());
-          if !argument_range.range_contains(&argument_count) {
-            return Err(Error::ArgumentCountMismatch {
-              recipe: recipe.name(),
-              parameters: recipe.parameters.clone(),
-              found: tail.len(),
-              min: recipe.min_arguments(),
-              max: recipe.max_arguments(),
-            });
-          }
-          grouped.push((recipe, &tail[0..argument_count]));
-          tail = &tail[argument_count..];
-        }
+    while let Some((first, mut rest)) = remaining.split_first() {
+      if let Some((invocation, consumed)) = self.invocation(
+        0,
+        &mut Vec::new(),
+        &arena,
+        &mut scopes,
+        config,
+        &dotenv,
+        search,
+        &scope,
+        first,
+        rest,
+      )? {
+        rest = &rest[consumed..];
+        invocations.push(invocation);
       } else {
-        missing.push((*argument).to_owned());
+        missing.push((*first).to_owned());
       }
-      rest = tail;
+      remaining = rest;
     }
 
     if !missing.is_empty() {
@@ -250,16 +265,23 @@ impl<'src> Justfile<'src> {
       });
     }
 
-    let context = RecipeContext {
-      settings: &self.settings,
-      config,
-      scope,
-      search,
-    };
-
     let mut ran = BTreeSet::new();
-    for (recipe, arguments) in grouped {
-      Self::run_recipe(&context, recipe, arguments, &dotenv, search, &mut ran)?;
+    for invocation in invocations {
+      let context = RecipeContext {
+        settings: invocation.settings,
+        config,
+        scope: invocation.scope,
+        search,
+      };
+
+      Self::run_recipe(
+        &context,
+        invocation.recipe,
+        invocation.arguments,
+        &dotenv,
+        search,
+        &mut ran,
+      )?;
     }
 
     Ok(())
@@ -277,6 +299,98 @@ impl<'src> Justfile<'src> {
       .or_else(|| self.aliases.get(name).map(|alias| alias.target.as_ref()))
   }
 
+  #[allow(clippy::too_many_arguments)]
+  fn invocation<'run>(
+    &'run self,
+    depth: usize,
+    path: &mut Vec<&'run str>,
+    arena: &'run Arena<Scope<'src, 'run>>,
+    scopes: &mut BTreeMap<Vec<&'run str>, &'run Scope<'src, 'run>>,
+    config: &'run Config,
+    dotenv: &'run BTreeMap<String, String>,
+    search: &'run Search,
+    parent: &'run Scope<'src, 'run>,
+    first: &'run str,
+    rest: &'run [&'run str],
+  ) -> RunResult<'src, Option<(Invocation<'src, 'run>, usize)>> {
+    if let Some(module) = self.modules.get(first) {
+      path.push(first);
+
+      let scope = if let Some(scope) = scopes.get(path) {
+        scope
+      } else {
+        let scope = module.scope(config, dotenv, search, &BTreeMap::new(), parent)?;
+        let scope = arena.alloc(scope);
+        scopes.insert(path.clone(), scope);
+        scopes.get(path).unwrap()
+      };
+
+      if rest.is_empty() {
+        if let Some(recipe) = &module.default {
+          recipe.check_can_be_default_recipe()?;
+          return Ok(Some((
+            Invocation {
+              settings: &module.settings,
+              recipe,
+              arguments: &[],
+              scope,
+            },
+            depth,
+          )));
+        }
+        Err(Error::NoDefaultRecipe)
+      } else {
+        module.invocation(
+          depth + 1,
+          path,
+          arena,
+          scopes,
+          config,
+          dotenv,
+          search,
+          scope,
+          rest[0],
+          &rest[1..],
+        )
+      }
+    } else if let Some(recipe) = self.get_recipe(first) {
+      if recipe.parameters.is_empty() {
+        Ok(Some((
+          Invocation {
+            arguments: &[],
+            recipe,
+            scope: parent,
+            settings: &self.settings,
+          },
+          depth,
+        )))
+      } else {
+        let argument_range = recipe.argument_range();
+        let argument_count = cmp::min(rest.len(), recipe.max_arguments());
+        if !argument_range.range_contains(&argument_count) {
+          return Err(Error::ArgumentCountMismatch {
+            recipe: recipe.name(),
+            parameters: recipe.parameters.clone(),
+            found: rest.len(),
+            min: recipe.min_arguments(),
+            max: recipe.max_arguments(),
+          });
+        }
+        Ok(Some((
+          Invocation {
+            arguments: &rest[..argument_count],
+            recipe,
+            scope: parent,
+            settings: &self.settings,
+          },
+          depth + argument_count,
+        )))
+      }
+    } else {
+      Ok(None)
+    }
+  }
+
   fn run_recipe(
     context: &RecipeContext<'src, '_>,
     recipe: &Recipe<'src>,
@@ -305,7 +419,7 @@ impl<'src> Justfile<'src> {
       dotenv,
       &recipe.parameters,
       arguments,
-      &context.scope,
+      context.scope,
       context.settings,
       search,
     )?;
