@@ -328,6 +328,20 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
             self.presume_keyword(Keyword::Export)?;
             items.push(Item::Assignment(self.parse_assignment(true)?));
           }
+          Some(Keyword::Import) if self.next_are(&[Identifier, StringToken]) => {
+            self.presume_keyword(Keyword::Import)?;
+            items.push(Item::Import {
+              relative: self.parse_string_literal()?,
+              absolute: None,
+            });
+          }
+          Some(Keyword::Mod) if self.next_are(&[Identifier, Identifier]) => {
+            self.presume_keyword(Keyword::Mod)?;
+            items.push(Item::Mod {
+              name: self.parse_name()?,
+              absolute: None,
+            });
+          }
           Some(Keyword::Set)
             if self.next_are(&[Identifier, Identifier, ColonEquals])
               || self.next_are(&[Identifier, Identifier, Comment, Eof])
@@ -350,9 +364,6 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
             }
           }
         }
-      } else if self.next_is(Bang) {
-        let directive = self.parse_directive()?;
-        items.push(directive);
       } else if self.accepted(At)? {
         let doc = pop_doc_comment(&mut items, eol_since_last_comment);
         items.push(Item::Recipe(self.parse_recipe(
@@ -778,24 +789,6 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
     Ok(value)
   }
 
-  fn parse_directive(&mut self) -> CompileResult<'src, Item<'src>> {
-    self.presume(Bang)?;
-    let name = self.expect(Identifier)?;
-    match name.lexeme() {
-      "include" => {
-        if let Some(include_line) = self.accept(Text)? {
-          Ok(Item::Include {
-            relative: include_line.lexeme().trim(),
-            absolute: None,
-          })
-        } else {
-          Err(self.error(CompileErrorKind::IncludeMissingPath)?)
-        }
-      }
-      directive => Err(name.error(CompileErrorKind::UnknownDirective { directive })),
-    }
-  }
-
   /// Parse a setting
   fn parse_set(&mut self) -> CompileResult<'src, Set<'src>> {
     self.presume_keyword(Keyword::Set)?;
@@ -1981,9 +1974,9 @@ mod tests {
   }
 
   test! {
-    name: include_directive,
-    text: "!include      some/file/path.txt     \n",
-    tree: (justfile (include "some/file/path.txt")),
+    name: import,
+    text: "import \"some/file/path.txt\"     \n",
+    tree: (justfile (import "some/file/path.txt")),
   }
 
   error! {
@@ -2076,7 +2069,7 @@ mod tests {
     column: 0,
     width:  1,
     kind: UnexpectedToken {
-      expected: vec![At, Bang, BracketL, Comment, Eof, Eol, Identifier],
+      expected: vec![At, BracketL, Comment, Eof, Eol, Identifier],
       found: BraceL,
     },
   }
@@ -2428,16 +2421,4 @@ mod tests {
       expected: 3..3,
     },
   }
-
-  error! {
-    name: unknown_directive,
-    input: "!inclood",
-    offset: 1,
-    line: 0,
-    column: 1,
-    width: 7,
-    kind: UnknownDirective {
-      directive: "inclood"
-    },
-  }
 }
